//*************************************************************************
//Declarations and Global variables for Adjunct Tables (programmed by NXH)*
//(version 3.0)                              				  *
//*************************************************************************
#ifndef __ADTABLE__
#define __ADTABLE__
#include<iostream>
#include<iomanip>
#include<cstring>
#include<cassert>
#include<cstdlib>
#include "../gconst.h"
#include "../random/random.h"
#include "../stree/stree.h"
#include "../lexicon/lexicon.h"


#ifdef _SP_MODE
// smart pointer is applied for debug

#include "../smart_ptr.h"

using namespace std;
//********************************************************
//Data structure and operations for list of size numbers *
//********************************************************
struct _snode {
	SIZE id;
	double prob; //probability
	boost::shared_ptr<_snode> next;
};
typedef struct _snode snode;
typedef boost::shared_ptr<_snode> sp_snode;
struct _slist {
	sp_snode head;
	SIZE card;
};
typedef struct _slist slist;
//*****************************
//Initialize a list of numbers*
//*****************************
void IniSlist(slist &s);
//***************************
//Check if the list is emty *
//***************************
BOOL EmptySlist(slist s);
//***********************************
//Return the cardinality of the list*
//***********************************
SIZE CardSlist(slist s);
//******************************
//Destroy a list               *
//******************************
void DestroySlist(slist &s);
//*****************************
//Add to the head of a list   *
//*****************************
void AddHeadSlist(slist &s, SIZE k, double prob);
//*********************************************
//This function deletes the node p in the list*
//*********************************************
void DeleteNodeSlist(slist &s, sp_snode p);
//************************
//Normalise Probability  *
//************************
void NormaliseTD(slist s);
//**************************************************
//Get a tree-id from the source list according to  *
//the probability distribution                     *
//**************************************************
SIZE GetTreeID(slist s);
//********************************
//This function display a list   *
//********************************
void DisplaySlist(slist s);
//******************************************************************
//Data structures and operations for address and address list      *
//******************************************************************
struct _address {
	slist source;
	sp_node addr;
	sp_node parent;
	unsigned int constraint;
	boost::shared_ptr<_address> next;
};
typedef struct _address address;
typedef boost::shared_ptr<_address> sp_address;
struct _addlist {
	sp_address head;
	BYTE card;
};
typedef struct _addlist addlist;
typedef boost::shared_array<_addlist> sa_addlist;
//***********************************************
//this function initialize the list of address  *
//***********************************************
void IniAddList(addlist &al);
//***********************************************
//This function check whether the adlist is emty*
//***********************************************
BOOL EmptyAddList(addlist al);
//*********************************************
//This function destroy the addlist           *
//*********************************************
void DestroyAddList(addlist &al);
//*****************************
//Add to the head of a list   *
//*****************************
void AddHeadAddList(addlist &al, slist s, sp_node addr, sp_node parent,
		unsigned int constr);
void AddTailAddList(addlist &al, slist s, sp_node addr, sp_node parent,
		unsigned int constr);
//*********************************************
//This function deletes the node p in the list*
//*********************************************
void DeleteNodeAddList(addlist &al, sp_address p);
//********************************
//This function display a list   *
//********************************
void DisplayAddList(addlist al);
//****************************************************
//declarations and operations for the adjunct table  *
//****************************************************
extern sa_addlist adtable;
extern SIZE entrynum;
//**********************************************
//this function initializes the adjunct table  *
//**********************************************
void InitializeAdTable(void);
//*************************************************************
//Add an address to an adjunction table entry from a tree     *
//*************************************************************
void AddAddress(addlist &al, sp_node t, sp_node parent, unsigned int constr);
//*****************************************************
//This function build i-th entry in the adjunct table *
//*****************************************************
void AddEntry(addlist &al, sp_node t, sp_node parent);
void NormaliseAD(void);
//*******************************************
//Build the adjunction table                *
//*******************************************
void BuildAdTable(void);


#else
// original xuan's code

using namespace std;
//********************************************************
//Data structure and operations for list of size numbers *
//********************************************************
struct _snode {
	SIZE id;
	double prob; //probability
	struct _snode *next;
};
typedef struct _snode snode;
struct _slist {
	snode *head;
	SIZE card;
};
typedef struct _slist slist;
//*****************************
//Initialize a list of numbers*
//*****************************
void IniSlist(slist &s);
//***************************
//Check if the list is emty *
//***************************
BOOL EmptySlist(slist s);
//***********************************
//Return the cardinality of the list*
//***********************************
SIZE CardSlist(slist s);
//******************************
//Destroy a list               *
//******************************
void DestroySlist(slist &s);
//*****************************
//Add to the head of a list   *
//*****************************
void AddHeadSlist(slist &s, SIZE k, double prob);
//*********************************************
//This function deletes the node p in the list*
//*********************************************
void DeleteNodeSlist(slist &s, snode *p);
//************************
//Normalise Probability  *
//************************
void NormaliseTD(slist s);
//**************************************************
//Get a tree-id from the source list according to  *
//the probability distribution                     *
//**************************************************
SIZE GetTreeID(slist s);
//********************************
//This function display a list   *
//********************************
void DisplaySlist(slist s);
//******************************************************************
//Data structures and operations for address and address list      *
//******************************************************************
struct _address {
	slist source;
	node *addr;
	node *parent;
	unsigned int constraint;
	struct _address *next;
};
typedef struct _address address;
struct _addlist {
	address *head;
	BYTE card;
};
typedef struct _addlist addlist;
//***********************************************
//this function initialize the list of address  *
//***********************************************
void IniAddList(addlist &al);
//***********************************************
//This function check whether the adlist is emty*
//***********************************************
BOOL EmptyAddList(addlist al);
//*********************************************
//This function destroy the addlist           *
//*********************************************
void DestroyAddList(addlist &al);
//*****************************
//Add to the head of a list   *
//*****************************
void AddHeadAddList(addlist &al, slist s, node *addr, node *parent,
		unsigned int constr);
void AddTailAddList(addlist &al, slist s, node *addr, node *parent,
		unsigned int constr);
//*********************************************
//This function deletes the node p in the list*
//*********************************************
void DeleteNodeAddList(addlist &al, address *p);
//********************************
//This function display a list   *
//********************************
void DisplayAddList(addlist al);
//****************************************************
//declarations and operations for the adjunct table  *
//****************************************************
extern addlist *adtable;
extern SIZE entrynum;
//**********************************************
//this function initializes the adjunct table  *
//**********************************************
void InitializeAdTable(void);
//*************************************************************
//Add an address to an adjunction table entry from a tree     *
//*************************************************************
void AddAddress(addlist &al, node *t, node *parent, unsigned int constr);
//*****************************************************
//This function build i-th entry in the adjunct table *
//*****************************************************
void AddEntry(addlist &al, node *t, node *parent);
void NormaliseAD(void);
//*******************************************
//Build the adjunction table                *
//*******************************************
void BuildAdTable(void);
#endif
#endif
