/*
 * variable.cpp
 *
 *  Created on: 2010. 5. 11.
 *      Original Author: Nguyen Xuan Hoai
 *      Modifications: Structural Complexity Lab, Seoul National U
 */

#include "dtree.h"

#ifdef _SP_MODE
void decodeTree(sp_dnode & p, stree beta, BOOL stochastic, sp_address s,
		addlist & newadd, BYTE i, stree & gama) {
	sp_dnode q, t;
#else
void decodeTree(dnode * & p, stree beta, BOOL stochastic, address * s, addlist & newadd, BYTE i, stree & gama) {
	dnode *q, *t;
#endif
	q = p->children;
	while (q->sibling) {
		q = q->sibling;
	}
	while (q->id <= -numetree) {
		t = q->children;
		while (t->sibling)
			t = t->sibling;
		q = t;
	}
	if (q->id >= 0) {
		beta = DecodeDTree(q, stochastic);
		s = newadd.head;
		i = 0;
		while (i < p->addno) {
			s = s->next;
			i++;
		}
		gama = AdjointTree(beta, gama, s->addr, s->parent);
	}
}

#ifdef _SP_MODE
stree decodeTree(sp_dnode t, BOOL stochastic) {
	sp_dnode p;
	sp_address s;
#else
stree decodeTree(dnode *t, BOOL stochastic) {
	address *s;
	dnode *p;
#endif

	stree beta, gama;
	addlist newadd;
	BYTE i;

	IniAddList(newadd);
	gama = CopyTree2(t->id, newadd);
	//substitue
	if (!EmptyLList(t->lexlist)) {
		if (stochastic) {
			//random substitution
			SubstituteLexicon(gama.root);
		} else {
			DSubstituteLexicon(gama.root, t->lexlist);
		}
	}

	p = t->children;
	while (p) {
		if (p->id >= 0) {
			//if the subtree is not currently switched off (subcode type1);
			beta = DecodeDTree(p, stochastic);
			//find the address and parent
			s = newadd.head;
			i = 0;
			while (i < p->addno) {
				s = s->next;
				i++;
			}
			gama = AdjointTree(beta, gama, s->addr, s->parent);
		} else if (p->id <= -numetree) {
			decodeTree(p, beta, stochastic, s, newadd, i, gama);
		}

		p = p->sibling;
	}

	DestroyAddList(newadd);
	return gama;
}

#ifdef _SP_MODE
stree DecodeDTree(sp_dnode t, BOOL stochastic) {
#else
	stree DecodeDTree(dnode *t, BOOL stochastic) {
#endif
	stree st;

	if (t->children == NULL) {
		st = CopyTree(etree[t->id]);//if leaves then just copy
		if (!EmptyLList(t->lexlist)) {
			if (stochastic) {//random substitution
				SubstituteLexicon(st.root);
			} else {
				DSubstituteLexicon(st.root, t->lexlist);
			}
		}
		return st;
	} else {
		stree gama = decodeTree(t, stochastic);
		return gama;
	}
}
