/*
 * DNodOrder.cpp
 *
 *  Created on: 2010. 5. 12.
 *      Original Author: Nguyen Xuan Hoai
 *      Modifications: Structural Complexity Lab, Seoul National U
 */
#include "dtree.h"

#ifdef _SP_MODE
void delDNode(sp_dnode & parent, sp_dnode & p, sp_dnode & t) {
#else
void delDNode(dnode * & parent, dnode * & p, dnode * & t) {
#endif
	//type sub-code
	if (parent) {
		//not pruning at root
		p = parent->children;
		if (p == t) {
			//if p is the first child
			parent->children = p->sibling;
		} else {
			while (p->sibling != t) {
				p = p->sibling;
			}
			p->sibling = t->sibling;
		}

		DeleteDNode(t);
	}
}

#ifdef _SP_MODE
void PruneDNode(sp_dnode & p, sp_dnode & t, sp_dnode & q, sp_dnode parent) {
#else
void PruneDNode(dnode * & p, dnode * & t, dnode * & q, dnode * parent) {
#endif
	//type2 sub-code
	p = t->children;
	while (p->sibling) {
		q = p->sibling;
		DeleteDNode(p);
		p = q;
	}
	//delete the node
	p->addno = t->addno;
	q = parent->children;
	if (q == t) {
		parent->children = p;
	} else {
		while (q->sibling != t) {
			q = q->sibling;
		}
		q->sibling = p;
	}

	p->sibling = t->sibling;
	DeleteSDNode(t);
	PruneDNode(p, parent);
}

#ifdef _SP_MODE
void PruneDNode(sp_dnode p, sp_dnode t, sp_dnode q){
#else
void PruneDNode(dnode * p, dnode * t, dnode * q){
#endif

    p = t->children;
    while(p){
        q = p->sibling;
        PruneDNode(p, t);
        p = q;
    }
}

#ifdef _SP_MODE
void PruneDNode(sp_dnode t, sp_dnode parent) {
	sp_dnode p, q;
#else
void PruneDNode(dnode *t, dnode *parent) {
	dnode *p, *q;
#endif
	if (t) {
		if (t->id < 0) { //if marked off
			if (t->id > -numetree) {
				delDNode(parent, p, t);
			} else {
				PruneDNode(p, t, q, parent);
			}
		} else {
			PruneDNode(p, t, q);
		}
	}
}

#ifdef _SP_MODE
void PruneDNode(sp_dnode t, int * temp_parent) {// to receive Null value as a pointer value
	assert(temp_parent == NULL);
	sp_dnode parent;
	sp_dnode p, q;

	if (t) {
		if (t->id < 0) { //if marked off
			if (t->id > -numetree) {
				delDNode(parent, p, t);
			} else {
				PruneDNode(p, t, q, parent);
			}
		} else {
			PruneDNode(p, t, q);
		}
	}
}
#endif
