//****************************************************************************
//This file contains declarations and operations for derivation trees in TAGs*
//Programmed by NXH (version 3.2)                                            *
//****************************************************************************
#ifndef __DTREE__
#define __DTREE__
#include<iostream>
#include<cstdlib>
#include<cstring>
#include<cassert>
#include "../gconst.h"
#include "../random/random.h"
#include "../stree/stree.h"
#include "../lexicon/lexicon.h"
#include "../adtable/adtable.h"


#ifdef _SP_MODE
#include "../smart_ptr.h"
// smart pointer is applied for debugging

//node structure for derivation tree node
struct _dnode {
	SIZE id;//the elementary tree id
	BYTE addno;//address number
	boost::shared_ptr<_dnode> children, sibling;//the addresses should be incresasing
	llist lexlist;//list of lexicon for substitution
};
typedef struct _dnode dnode;
typedef boost::shared_ptr<_dnode> sp_dnode;
//Operations for derivation tree
//***************************
//Make a Node               *
//***************************
sp_dnode MakeDNode(SIZE id, BYTE addno);
void DeleteSDNode(sp_dnode t);

//*****************************
//Delete a subtree            *
//*****************************
void DeleteDNode(sp_dnode t);

//**********************************
//Copy a subtree                   *
//**********************************
sp_dnode CopyDNode(sp_dnode s);

//****************************************
//Compare two trees (structure only)     *
//****************************************
BOOL DNodeCmp(sp_dnode t, sp_dnode s);
unsigned int GetHeightDNode(sp_dnode t);
void DisplayDTree(sp_dnode r);

//*********************************************************
//Travel a Tree in Preorder,Inorder and Postoder          *
//*********************************************************
extern void (*_doitdnode)(sp_dnode n);
void PreorderDNode(sp_dnode t);

//Travel a Tree in Inoder
void InorderDNode(sp_dnode t);

//Travel a Tree in Postorder
void PostorderDNode(sp_dnode t);

//**********************************
//Travel the tre level by level    *
//**********************************
//We need a queue
struct _queuenode {
	sp_dnode id;
	boost::shared_ptr<_queuenode> next;
};
typedef struct _queuenode queuenode;
typedef boost::shared_ptr<_queuenode> sp_queuenode;
struct _queue {
	sp_queuenode head;
	sp_queuenode tail;
	unsigned int card;
};
typedef struct _queue queue;
typedef boost::shared_ptr<_queue> sp_queue;
void IniQueue(queue &q);
BOOL EmptyQueue(queue q);
void AddQueue(queue &q, sp_dnode t);
sp_dnode PopQueue(queue &q);
void LevelorderDNode(sp_dnode t);

//********************************
//Compute the size of a subtree  *
//********************************
extern unsigned int _dnodecount;
extern BOOL _countleave;
void DNodeCount(sp_dnode t);
unsigned int GetSizeDNode(sp_dnode t, BOOL countleave);

//***************************************
//Get the average branching of a dnode  *
//***************************************
extern unsigned int _dnodebranch;
void DNodeBranchCount(sp_dnode t);
double GetAVGDNode(sp_dnode t);

//****************************************************
//Check if dnode p is in subtree stemed from dnode q *
//****************************************************
extern BOOL _indnode;
extern sp_dnode _checkdnode;
void CheckDNode(sp_dnode s);
BOOL InDNode(sp_dnode p, sp_dnode q);
int DNodeOrder(sp_dnode p, sp_dnode q);

//*******************************************************
//Find ith node and its parent (in preorder search)     *
//*******************************************************
extern sp_dnode _idnode;
extern sp_dnode _idnodeparent;
extern unsigned int _indexdnode;
void FindDNode(sp_dnode t, sp_dnode parent);
void LocateDNode(unsigned int i, sp_dnode t, sp_dnode parent);

// for Null input of LocateDNode at third arguement
void LocateDNode(unsigned int i, sp_dnode t, int * parent);

//*************************************************************
//This function copy a sub node  tree and a list of address   *
//*************************************************************
extern sp_node _oldfoot2;//this variable contains the foot node of the source tree
extern sp_node _newfoot2;//this variable contains the footnode of the newtree
sp_node CopyNode2(sp_node s, sp_node parent, SIZE treeindex, addlist &newadd);
stree CopyTree2(SIZE treeid, addlist &newadd);

//*************************************************************
//This function decode a derived tree from a derivation tree  *
//Using Weir Function and substitution                        *
//*************************************************************
stree DecodeDTree(sp_dnode t, BOOL stochastic);

//***********************************************************
//Randomly add one more beta tree in to the derivation tree *
//***********************************************************
extern unsigned int _opendnodeno;
extern sp_dnode _opendnode;
extern BOOL _countorfind;
void CountOpenDNode(sp_dnode s);

//***********************************************************************************
//This function is used for counting/finding the number of open adjoining addresses *
//***********************************************************************************
extern int _openaddno; //can't be unsigned int because of subtraction
extern sp_dnode _opendnodeadd;
extern BYTE _openaddindex;
extern BOOL _cf, _stop;
void CountOpenAddDNode(sp_dnode s);

//Counting Open Addresses
unsigned int CountOpenAdd(sp_dnode s);

//locating an open address
sp_dnode LocateOpenAdd(sp_dnode s, unsigned int index, BYTE &add);
BOOL AddBeta(sp_dnode s);

//*******************************************************
//This function add new beta tree in a full shape maner *
//*******************************************************
BOOL AddFullBeta(sp_dnode s);

//***************************************
//This Function delete randomly a leave *
//***************************************
extern unsigned int _leavecount;
void CountLeave(sp_dnode s);

void DeleteLeave(sp_dnode s, sp_dnode parent);

BOOL DeleteBeta(sp_dnode s);

//**************************************************************
//Pruning tree by deleting marked subtrees                     *
//(subtrees that rooted with negative id)(added inversion 3.1) *
//**************************************************************
void PruneDNode(sp_dnode t, sp_dnode parent);
void PruneDNode(sp_dnode t, int * temp_parent); // to recevie NULL value by the second parameter

//**************************************************************
//These fucntions are used to unmark the genes in one genotype *
//**************************************************************
void UnMarkingDNode(sp_dnode t);

void UnMarkingDtree(sp_dnode t);

#else
// origianl xuan's code

//node structure for derivation tree node
struct _dnode {
	SIZE id;//the elementary tree id
	BYTE addno;//address number
	struct _dnode *children, *sibling;//the addresses should be incresasing
	llist lexlist;//list of lexicon for substitution
};
typedef struct _dnode dnode;
//Operations for derivation tree
//***************************
//Make a Node               *
//***************************
dnode *MakeDNode(SIZE id, BYTE addno);
void DeleteSDNode(dnode *t);

//*****************************
//Delete a subtree            *
//*****************************
void DeleteDNode(dnode *t);

//**********************************
//Copy a subtree                   *
//**********************************
dnode *CopyDNode(dnode *s);

//****************************************
//Compare two trees (structure only)     *
//****************************************
BOOL DNodeCmp(dnode *t, dnode *s);
unsigned int GetHeightDNode(dnode *t);
void DisplayDTree(dnode *r);

//*********************************************************
//Travel a Tree in Preorder,Inorder and Postoder          *
//*********************************************************
extern void (*_doitdnode)(dnode *n);
void PreorderDNode(dnode *t);

//Travel a Tree in Inoder
void InorderDNode(dnode *t);

//Travel a Tree in Postorder
void PostorderDNode(dnode *t);

//**********************************
//Travel the tre level by level    *
//**********************************
//We need a queue
struct _queuenode {
	dnode *id;
	struct _queuenode *next;
};
typedef struct _queuenode queuenode;
struct _queue {
	queuenode *head;
	queuenode *tail;
	unsigned int card;
};
typedef struct _queue queue;
void IniQueue(queue &q);
BOOL EmptyQueue(queue q);
void AddQueue(queue &q, dnode *t);
dnode *PopQueue(queue &q);
void LevelorderDNode(dnode *t);

//********************************
//Compute the size of a subtree  *
//********************************
extern unsigned int _dnodecount;
extern BOOL _countleave;
void DNodeCount(dnode *t);
unsigned int GetSizeDNode(dnode *t, BOOL countleave);

//***************************************
//Get the average branching of a dnode  *
//***************************************
extern unsigned int _dnodebranch;
void DNodeBranchCount(dnode *t);
double GetAVGDNode(dnode *t);

//****************************************************
//Check if dnode p is in subtree stemed from dnode q *
//****************************************************
extern BOOL _indnode;
extern dnode *_checkdnode;
void CheckDNode(dnode *s);
BOOL InDNode(dnode *p, dnode *q);
int DNodeOrder(dnode *p, dnode *q);

//*******************************************************
//Find ith node and its parent (in preorder search)     *
//*******************************************************
extern dnode *_idnode;
extern dnode *_idnodeparent;
extern unsigned int _indexdnode;
void FindDNode(dnode *t, dnode *parent);
void LocateDNode(unsigned int i, dnode *t, dnode *parent);

//*************************************************************
//This function copy a sub node  tree and a list of address   *
//*************************************************************
extern node *_oldfoot2;//this variable contains the foot node of the source tree
extern node *_newfoot2;//this variable contains the footnode of the newtree
node *CopyNode2(node *s, node *parent, SIZE treeindex, addlist &newadd);
stree CopyTree2(SIZE treeid, addlist &newadd);

//*************************************************************
//This function decode a derived tree from a derivation tree  *
//Using Weir Function and substitution                        *
//*************************************************************
stree DecodeDTree(dnode *t, BOOL stochastic);

//***********************************************************
//Randomly add one more beta tree in to the derivation tree *
//***********************************************************
extern unsigned int _opendnodeno;
extern dnode *_opendnode;
extern BOOL _countorfind;
void CountOpenDNode(dnode *s);

//***********************************************************************************
//This function is used for counting/finding the number of open adjoining addresses *
//***********************************************************************************
extern int _openaddno; //can't be unsigned int because of subtraction
extern dnode *_opendnodeadd;
extern BYTE _openaddindex;
extern BOOL _cf, _stop;
void CountOpenAddDNode(dnode *s);

//Counting Open Addresses
unsigned int CountOpenAdd(dnode *s);

//locating an open address
dnode *LocateOpenAdd(dnode *s, unsigned int index, BYTE &add);
BOOL AddBeta(dnode *s);

//*******************************************************
//This function add new beta tree in a full shape maner *
//*******************************************************
BOOL AddFullBeta(dnode *s);

//***************************************
//This Function delete randomly a leave *
//***************************************
extern unsigned int _leavecount;
void CountLeave(dnode *s);

void DeleteLeave(dnode *s, dnode *parent);

BOOL DeleteBeta(dnode *s);

//**************************************************************
//Pruning tree by deleting marked subtrees                     *
//(subtrees that rooted with negative id)(added inversion 3.1) *
//**************************************************************
void PruneDNode(dnode *t, dnode *parent);

//**************************************************************
//These fucntions are used to unmark the genes in one genotype *
//**************************************************************
void UnMarkingDNode(dnode *t);

void UnMarkingDtree(dnode *t);

#endif
#endif
