/*
 * PointReplace.cpp
 *
 *  Created on: 2010. 5. 12.
 *      Original Author: Nguyen Xuan Hoai
 *      Modifications: Structural Complexity Lab, Seoul National U
 */
#include "genetic.h"
#include "../util/util.h"

#ifdef _SP_MODE
void checkCombaibility(sp_address & p, sp_dnode parentdnode1, unsigned int & h,
		sp_dnode & dnode1, sp_snode & t, BOOL & found1, SIZE & i, BOOL & found)
#else
void checkCombaibility(address * & p, dnode * parentdnode1, unsigned int & h, dnode * & dnode1, snode * & t, BOOL & found1, SIZE & i, BOOL & found)
#endif
{
	p = adtable[parentdnode1->id].head;
	h = 0;
	while (h < dnode1->addno) {
		p = p->next;
		h++;
	}
	t = (p->source).head;
	found1 = FALSE;
	while (t && !found1) {
		if (t->id == i) {
			found1 = TRUE;
		} else {
			t = t->next;
		}
	}
	if (!found1) {
		found = TRUE;
	}
}

#ifdef _SP_MODE
void checkCombaibility(sp_address p, SIZE & i, unsigned int & h, sp_dnode & q,
		sp_snode t, BOOL found1, BOOL & found)
#else
void checkCombaibility(address * p, SIZE & i, unsigned int & h, dnode * & q, snode * t, BOOL found1, BOOL & found)
#endif
{
	//check if q->id is in the source list
	//find the address
	p = adtable[i].head;
	h = 0;
	while (p && (h < q->addno)) {
		p = p->next;
		h++;
	}
	t = (p->source).head;
	found1 = FALSE;
	while (t && !found1) {
		if (t->id == q->id) {
			found1 = TRUE;
		} else {
			t = t->next;
		}
	}
	if (!found1) {
		found = TRUE;
	}
}

BOOL PointReplace(individual &child) {
	unsigned int j, k, h;
	SIZE i;
	#ifdef _SP_MODE
		sp_dnode q, dnode1, parentdnode1;
		sp_address p;
		sp_snode t;
		boost::shared_array<BOOL> check;
	#else
		dnode *q, *dnode1, *parentdnode1;
		address *p;
		snode *t;
		BOOL *check;
	#endif
	BOOL found, found1;
	unsigned int replacepoint;
	check = new BOOL[numetree];
	assert(check!=NULL);
	j = 0;

	while (j < MAXATEMPT) {
		getCrossoverPoints(replacepoint, child, dnode1);
		parentdnode1 = _idnodeparent;
		for (i = 0; i < numetree; i++) {
			check[i] = FALSE;
		}

		k = 0;
		for (i = 0; (i < numetree) && (i != dnode1->id); i++) {
			found = FALSE;
			//the check compatibility with parent address
			if (parentdnode1) {
				checkCombaibility(p, parentdnode1, h, dnode1, t, found1, i,
						found);
			}

			q = dnode1->children;
			while (q && !found) {
				//check if tree i has address and tree q->id
				if (q->addno >= adtable[i].card) {//no such address
					found = TRUE;
				} else {
					checkCombaibility(p, i, h, q, t, found1, found);
				}
				q = q->sibling;
			}
			if (!found) {//this tree is a candidate
				check[i] = TRUE;
				k++;
			}
		}
		if (k > 0) {//there are some trees to replace
			k = IRandom(1, k); //choose at random
			h = 0;
			while (k > 0) {
				if (check[h]) {
					k--;
				}
				if (k > 0) {
					h++;
				}
			}
			dnode1->id = h;
			DeleteLList(dnode1->lexlist);
			dnode1->lexlist = BuildLList(etree[h].root);
			//deletele list
			#ifdef _SP_MODE
			check_ptr(check);
			#else
			delete[] check;
			#endif

			return TRUE;
		}
		j++;
	}
	return FALSE;
}
