/*
 * Relocate.cpp
 *
 *  Created on: 2010. 5. 12.
 *      Original Author: Nguyen Xuan Hoai
 *      Modifications: Structural Complexity Lab, Seoul National U
 */
#include "genetic.h"
#include "../util/util.h"

#ifdef _SP_MODE
void dettachNRelocate(sp_dnode p, sp_dnode parentdnode1, sp_dnode dnode1,
		sp_dnode dnode2, BOOL found, sp_dnode q)
#else
void dettachNRelocate(dnode * p, dnode * parentdnode1, dnode * dnode1, dnode * dnode2, BOOL found, dnode * q)
#endif
{
	//Deattach dnode1
	p = parentdnode1->children;
	if (p == dnode1) {
		parentdnode1->children = p->sibling;
	} else {
		while (p->sibling != dnode1) {
			p = p->sibling;
		}
		p->sibling = dnode1->sibling;
	}

	//Relocate dnode1
	p = dnode2->children;
	if (p == NULL) {
		dnode2->children = dnode1;
		dnode1->sibling = NULL;
	} else {
		found = FALSE;
		while (p && !found) {
			if (p->addno > dnode1->addno) {
				found = TRUE;
			} else {
				p = p->sibling;
			}
		}
		findNInsert(found, q, dnode2, p, dnode1);
	}
}

BOOL Relocate(individual &child) {
	#ifdef _SP_MODE
		sp_dnode dnode1, parentdnode1, dnode2, p, q;
		boost::shared_array<BOOL> check;
		sp_address a;
		sp_snode t;
	#else
		dnode *dnode1, *parentdnode1, *dnode2, *p, *q;
		BOOL *check;
		address *a;
		snode *t;
	#endif
	unsigned int j, relocatepoint, opendnodeno;
	BYTE i, k;
	BOOL found, act;
	_opendnodeno = 0;
	_countorfind = TRUE;//counting
	_doitdnode = CountOpenDNode;
	PreorderDNode(child.chrom);//result is in _opendnodeno

	if (_opendnodeno <= 0) {
		return FALSE;
	} else {
		opendnodeno = _opendnodeno;
	}

	j = 0;
	while (j < MAXATEMPT) {
		getCrossOverPoints(relocatepoint, child, dnode1, parentdnode1);
		findOpenNode(opendnodeno, child, dnode2);

		if (!InDNode(dnode2, dnode1)) {
			//check if dnode1 can be adjoined into dnode2
			//find open addresses

			check = new BOOL[adtable[dnode2->id].card];
			assert(check!=NULL);
			for (i = 0; i < adtable[dnode2->id].card; i++) {
				check[i] = FALSE;
			}

			act = FALSE;
			p = dnode2->children;
			while (p) {
				check[p->addno] = TRUE;
				p = p->sibling;
			}

			k = 0;
			a = adtable[dnode2->id].head;
			for (i = 0; i < adtable[dnode2->id].card; i++) {
				if (!check[i]) {
					checkAdjoint(t, a, found, dnode1);
					if (found) {
						k++;
					} else {
						check[i] = TRUE;
					}
				}
				a = a->next;
			}

			//choose the address to attach
			if (k > 0) {
				k = IRandom(1, k);
				i = 0;
				while (k > 0) {
					if (!check[i]) {
						k--;
					}
					if (k > 0) {
						i++;
					}
				}
				dnode1->addno = i;
				act = TRUE;
				dettachNRelocate(p, parentdnode1, dnode1, dnode2, found, q);
			}
			#ifdef _SP_MODE
			check_ptr(check);
			#else
			delete[] check;
			#endif
			if (act) {
				//recompute the branch factor
				child.branch = GetAVGDNode(child.chrom);
				return TRUE;
			}
		}
		j++;
	}
	return FALSE;
}
