/*
 * ReplaceSubTree.cpp
 *
 *  Created on: 2010. 5. 12.
 *      Original Author: Nguyen Xuan Hoai
 *      Modifications: Structural Complexity Lab, Seoul National U
 */
#include "genetic.h"
#include "../util/util.h"

#ifdef _SP_MODE
void recomputeSizeNBranch(individual & child, SIZE size1, SIZE size2,
		unsigned int l, sp_dnode dnode1)
#else
void recomputeSizeNBranch(individual & child, SIZE size1, SIZE size2, unsigned int l, dnode * dnode1)
#endif
{
	child.size = ABSDIFF(child.size+size1,size2);
	if (child.size > MAXSIZE)//randomly cut
	{
		for (l = MAXSIZE; l < child.size; l++) {
			DeleteBeta(dnode1);
		}
		child.size = MAXSIZE;
	}
	child.branch = GetAVGDNode(child.chrom);
}

void setSize(BOOL fairsize, SIZE & size1, SIZE size, SIZE & size2,
		unsigned int diff, unsigned int & k) {
	if (!fairsize) {
		size1 = size;
	} else if (Flip(0.5)) {
		size1 = size2 + IRandom(0, diff); //increase size or decrease size
	} else {
		k = IRandom(0, diff);
		if (size2 > k) {
			size1 = size2 - k;
		} else {
			size1 = size2 - IRandom(0, size2);
		}
	}

}

#ifdef _SP_MODE
void replaceTree(sp_dnode & temp, individual & child, unsigned int treeid,
		unsigned int & l, SIZE & size1, BOOL & shape)
#else
void replaceTree(dnode * & temp, individual & child, unsigned int treeid, unsigned int & l, SIZE & size1, BOOL & shape)
#endif
{
	temp = child.chrom;
	//choosing one alpha at random
	treeid = IRandom(1, numalpha);
	//find the alpha tree;
	l = 0;
	while (treeid > 0) {
		if (IsAlpha(etree[l])) {
			treeid--;
		}
		if (treeid > 0) {
			l++;
		}
	}

	child.chrom = GenShapeTree(size1, l, 0, shape);
	DeleteDNode(temp);
	child.size = size1;
	recomputeSizeNBranch(child, l);
}

#ifdef _SP_MODE
void addBetaTrees(sp_dnode & dnode2, SIZE & size1, unsigned int k,
		sp_dnode & dnode1, BOOL shape)
#else
void addBetaTrees(dnode * & dnode2, SIZE & size1, unsigned int k, dnode * & dnode1, BOOL shape)
#endif
{
	//adding beta trees
	dnode2 = GenShapeTree(size1, k, dnode1->addno, shape);
	dnode1->id = dnode2->id;
	dnode1->children = dnode2->children;
	dnode1->lexlist = dnode2->lexlist;
}

BOOL ReplaceSubTree(individual &child, SIZE size, BOOL shape, BOOL fairsize,
		unsigned int diff) {

	unsigned int replacepoint, j, k, l, treeid;
	SIZE size1, size2;
	#ifdef _SP_MODE
		sp_dnode parentdnode1, dnode1, dnode2, temp, q;
		sp_address p;
	#else
		dnode *parentdnode1, *dnode1, *dnode2, *temp, *q;
		address *p;
	#endif
	BYTE i;
	j = 0;

	while (j < MAXATEMPT) {
		replacepoint = IRandom(1, child.size);//include the root
		LocateDNode(replacepoint, child.chrom, NULL);
		dnode1 = _idnode;
		parentdnode1 = _idnodeparent;
		size2 = GetSizeDNode(dnode1, TRUE);
		setSize(fairsize, size1, size, size2, diff, k);
		if (replacepoint == 1) {//replace at root the generate totally new tree
			if (((ABSDIFF(child.size+size1,size2) <= MAXSIZE) || _cut)
					&& (ABSDIFF(child.size+size1,size2) >= MINSIZE)) {
				replaceTree(temp, child, treeid, l, size1, shape);
				return TRUE;
			}
		} else if (((ABSDIFF(child.size+size1,size2) <= MAXSIZE) || _cut)
				&& (ABSDIFF(child.size+size1,size2) >= MINSIZE)) { //not the root
			//find the replace tree for dnode
			//find the address
			p = adtable[parentdnode1->id].head;
			i = 0;
			while (i < dnode1->addno) {
				p = p->next;
				i++;
			}
			//randomly choose a tree
			k = GetTreeID(p->source);

			q = dnode1->children;
			addBetaTrees(dnode2, size1, k, dnode1, shape);
			#ifdef _SP_MODE
				check_ptr(dnode2);
			#else
				delete dnode2;
			#endif
			recomputeSizeNBranch(child, size1, size2, l, dnode1);
			//Delete old chidren of dnode1;
			while (q) {
				temp = q;
				q = q->sibling;
				DeleteDNode(temp);
			}
			return TRUE;
		}
		j++;
	}

	return FALSE;
}
