/*
 * Replicate.cpp
 *
 *  Created on: 2010. 5. 12.
 *      Original Author: Nguyen Xuan Hoai
 *      Modifications: Structural Complexity Lab, Seoul National U
 */
#include "genetic.h"
#include "../util/util.h"

#ifdef _SP_MODE
void chooseAddressNAttach(BYTE k, BYTE i, boost::shared_array<BOOL> & check,
		BOOL & act, sp_dnode temp, sp_dnode dnode1, sp_dnode p,
		sp_dnode dnode2, BOOL found, sp_dnode q)
#else
void chooseAddressNAttach(BYTE k, BYTE i, BOOL* check, BOOL & act, dnode * temp, dnode * dnode1, dnode * p, dnode * dnode2, BOOL found, dnode * q)
#endif
{
	k = IRandom(1, k);
	i = 0;
	while (k > 0) {
		if (!check[i]) {
			k--;
		}
		if (k > 0) {
			i++;
		}
	}

	act = TRUE;
	//copy dnode1
	temp = dnode1;
	dnode1 = CopyDNode(temp);
	dnode1->addno = i;
	dnode1->sibling = NULL;
	//Replicate dnode1
	p = dnode2->children;
	if (p == NULL) {
		dnode2->children = dnode1;
	} else {
		found = FALSE;
		while (p && !found) {
			if (p->addno > dnode1->addno) {
				found = TRUE;
			} else {
				p = p->sibling;
			}
		}
		findNInsert(found, q, dnode2, p, dnode1);
	}
}

BOOL Replicate(individual &child, BOOL recursive) {
#ifdef _SP_MODE
	sp_dnode dnode1, dnode2, p, q, temp;
	boost::shared_array<BOOL> check;
	sp_address a;
	sp_snode t;
#else
	dnode *dnode1, *dnode2, *p, *q, *temp;
	BOOL *check;
	address *a;
	snode *t;
	//find open node
#endif
	BOOL found, act;
	unsigned int j, relocatepoint, h, opendnodeno;
	SIZE size1;
	BYTE i, k;
	_opendnodeno = 0;
	_countorfind = TRUE;//counting
	_doitdnode = CountOpenDNode;
	PreorderDNode(child.chrom);//result is in _opendnodeno

	if (_opendnodeno <= 0) {//if there is no open nodes
		return FALSE;
	} else {
		opendnodeno = _opendnodeno;
	}

	j = 0;
	while (j < MAXATEMPT) {
		getCrossoverPoints(relocatepoint, child, dnode1);
		size1 = GetSizeDNode(dnode1, TRUE);

		if ((child.size + size1 <= MAXSIZE) || _cut) {
			findOpenNode(opendnodeno, child, dnode2);

			if (recursive || !InDNode(dnode2, dnode1)) {
				//check if dnode1 can be adjoined into dnode2
				//find open addresses
				act = FALSE;
				check = new BOOL[adtable[dnode2->id].card];
				assert(check!=NULL);
				for (i = 0; i < adtable[dnode2->id].card; i++) {
					check[i] = FALSE;
				}
				p = dnode2->children;
				while (p) {
					check[p->addno] = TRUE;
					p = p->sibling;
				}
				k = 0;
				a = adtable[dnode2->id].head;
				for (i = 0; i < adtable[dnode2->id].card; i++) {
					if (!check[i]) {
						checkAdjoint(t, a, found, dnode1);
						if (found) {
							k++;
						} else {
							check[i] = TRUE;
						}
					}
					a = a->next;
				}
				//choose the address to attach
				if (k > 0) {
					chooseAddressNAttach(k, i, check, act, temp, dnode1, p,
							dnode2, found, q);
				}
				#ifdef _SP_MODE
				check_ptr(check);
				#else
				delete[] check;
				#endif
				if (act) {
					//check size and cut
					child.size += size1;
					recomputeSizeNBranch(child, h);
					return TRUE;
				}
			}
		}
		j++;
	}
	return FALSE;
}
