/*
 * SubSubTreeSwap.cpp
 *
 *  Created on: 2010. 5. 12.
 *      Original Author: Nguyen Xuan Hoai
 *      Modifications: Structural Complexity Lab, Seoul National U
 */
#include "genetic.h"
#include "../util/util.h"

#ifdef _SP_MODE
void swapNode(sp_dnode temp, sp_dnode & dnode1, sp_dnode & dnode2)
#else
void swapNode(dnode * temp, dnode * & dnode1, dnode * & dnode2)
#endif
{
    temp = dnode1->children;
    dnode1->children = dnode2->children;
    dnode2->children = temp;
}

#ifdef _SP_MODE
void checkAdjoint(sp_dnode & q, sp_dnode & dnode1, BOOL & found1, sp_address & p, unsigned int & i, sp_snode & t, BOOL & found2)
#else
void checkAdjoint(dnode * & q, dnode * & dnode1, BOOL & found1, address * & p, unsigned int & i, snode * & t, BOOL & found2)
#endif
{
    if (q->addno >= adtable[dnode1->id].card) {
		found1 = FALSE;
	} else {
		checkAdjoint(p, dnode1, i, q, t, found2);
		if (!found2) {
			found1 = FALSE;
		}
	}
    q = q->sibling;
}

#ifdef _SP_MODE
void checkAdjoint(sp_dnode & q, sp_dnode & dnode2, BOOL & found1, sp_dnode & dnode1, sp_address & p, unsigned int & i, sp_snode & t, BOOL & found2)
#else
void checkAdjoint(dnode * & q, dnode * & dnode2, BOOL & found1, dnode * & dnode1, address * & p, unsigned int & i, snode * & t, BOOL & found2)
#endif
{
    q = dnode2->children;
    if(q)//check whether children nodes of dnode2 can be adjoined into dnode1
    {
        while(q && found1){
            checkAdjoint(q, dnode1, found1, p, i, t, found2);
        }
    }

}

BOOL SubSubTreeSwap(individual parent1, individual parent2, individual &child1,
		individual &child2, BOOL fairsize, unsigned int diff) {

	unsigned int crosspoint1, crosspoint2, j;
	SIZE size1, size2;
	#ifdef _SP_MODE
	sp_dnode dnode1, dnode2, q, temp;
	sp_address p;
	sp_snode t;
	#else
	dnode *dnode1, *dnode2, *q, *temp;
	address *p;
	snode *t;
	#endif
	BOOL found1, found2;
	unsigned int i;
	j = 0;

	while (j <= MAXATEMPT) {
		getCrossoverPoints(crosspoint1, parent1, dnode1);
		getCrossoverPoints(crosspoint2, parent2, dnode2);

		found1 = TRUE;
		//check whether children nodes of dnode2 can be adjoined into dnode1
		checkAdjoint(q, dnode2, found1, dnode1, p, i, t, found2);
		if (found1) {
			//check if children nodes of dnode1 can be adjoined into dnode2
			checkAdjoint(q, dnode1, found1, dnode2, p, i, t, found2);
		}
		if (found1) {
			//check size
			size1 = GetSizeDNode(dnode1, TRUE);
			size2 = GetSizeDNode(dnode2, TRUE);
			setAct(fairsize, size1, size2, diff, found1, parent1, parent2);

			if (found1) {
				swapNode(temp, dnode1, dnode2);
				recomputeSizeNBranch(child1, parent1, size2, size1, dnode1);
				recomputeSizeNBranch(child2, parent2, size1, size2, dnode2);

				return TRUE;
			}
		}
		j++;
	}
	return FALSE;
}
