/*
 * SubTreeSwap.cpp
 *
 *  Created on: 2010. 5. 12.
 *      Original Author: Nguyen Xuan Hoai
 *      Modifications: Structural Complexity Lab, Seoul National Univ
 */
#include "genetic.h"
#include "../util/util.h"

#ifdef _SP_MODE
void beforeSwapping(sp_dnode & dnode1, sp_dnode & ch2, sp_dnode & dnode2, sp_dnode & ch)
#else
void beforeSwapping(dnode * & dnode1, dnode * & ch2, dnode * & dnode2, dnode * & ch)
#endif
{
	//root adjunction at dnode1 and no root adjunction in dnode2 before swaping
	if (RootAdjoinability(dnode1->id)) {
		ch2 = dnode2->children;
		while (ch2->sibling) {
			ch2 = ch2->sibling;
		}
		ch = dnode1->children;
		if (ch) {
			while (ch->sibling) {
				ch = ch->sibling;
			}
			ch->sibling = ch2;
			ch2->addno = adtable[dnode1->id].card - 1;
		} else {
			dnode1->children = ch2;
			ch2->addno = adtable[dnode1->id].card - 1;
		}
		//cutting the old root
		ch = dnode2->children;
		if (ch == ch2) {
			dnode2->children = NULL;
		} else {
			while (ch->sibling != ch2) {
				ch = ch->sibling;
			}
			ch->sibling = NULL;
		}
	}
}

#ifdef _SP_MODE
void beforeSwapping(sp_dnode & ch, sp_dnode & dnode1, sp_dnode & ch2,sp_dnode & dnode2, unsigned int temp1, llist l, sp_dnode temp2)
#else
void beforeSwapping(dnode * & ch, dnode * & dnode1, dnode * & ch2, dnode * & dnode2, unsigned int temp1, llist l, dnode * temp2)
#endif
{
	//returning swap of  the root adjunctions
	ch = dnode1->children;
	while (ch->sibling) {
		ch = ch->sibling;
	}
	ch2 = dnode2->children;
	while (ch2->sibling) {
		ch2 = ch2->sibling;
	}

	//swaping back the root adjoining node
	temp1 = ch->id;
	ch->id = ch2->id;
	ch2->id = temp1;
	l = ch->lexlist;
	ch->lexlist = ch2->lexlist;
	ch2->lexlist = l;
	temp2 = ch->children;
	ch->children = ch2->children;
	ch2->children = temp2;
}

#ifdef _SP_MODE
void setSize(sp_dnode & ch, sp_dnode & dnode1, SIZE & size1)
#else
void setSize(dnode * & ch, dnode * & dnode1, SIZE & size1)
#endif
{
    ch = dnode1->children;
    while(ch->sibling){
        ch = ch->sibling;
    }
    size1 = size1 - GetSizeDNode(ch, TRUE);
}

#ifdef _SP_MODE
void calculateSize(BOOL & exroot1, BOOL & exroot2, BOOL root1, sp_dnode & dnode1, BOOL root2, sp_dnode & dnode2, BOOL & act, SIZE & size1, SIZE & size2)
#else
void calculateSize(BOOL & exroot1, BOOL & exroot2, BOOL root1, dnode * & dnode1, BOOL root2, dnode * & dnode2, BOOL & act, SIZE & size1, SIZE & size2)
#endif
{
    exroot1 = FALSE;
    exroot2 = FALSE;
    root1 = RootAdjoined(dnode1);
    root2 = RootAdjoined(dnode2);

    if (root1) {
		if (Flip(0.5)) {
			exroot1 = TRUE;
		}
	}
    if (root2) {
		if (Flip(0.5)) {
			exroot2 = TRUE;
		}
	}

    //can't swap if both have root and one don't swap root, one swap root
    if (exroot1 && root2 && !exroot2) {
		act = FALSE;
	}
    if (exroot2 && root1 && !exroot1) {
		act = FALSE;
	}

    //calculate the size of the two sub-trees
    size1 = GetSizeDNode(dnode1, TRUE);
    size2 = GetSizeDNode(dnode2, TRUE);
}

BOOL SubTreeSwap(individual parent1, individual parent2, individual &child1, individual &child2, BOOL fairsize, unsigned int diff) {

	unsigned int crosspoint1, crosspoint2, j, temp1;
	SIZE size1, size2;
	#ifdef _SP_MODE
		sp_dnode dnodeparent1, dnodeparent2, dnode1, dnode2, temp2, ch, ch2;
		sp_address p;
		sp_snode t;
	#else
		dnode *dnodeparent1, *dnodeparent2, *dnode1, *dnode2, *temp2, *ch, *ch2;
		address *p;
		snode *t;
	#endif
	BOOL found1, found2, act, exroot1, exroot2, root1, root2;
	unsigned int i;
	BYTE index;
	llist l;
	j = 0;

	while (j < MAXATEMPT) {
		getCrossOverPoints(crosspoint1, parent1, dnode1, dnodeparent1);
		getCrossOverPoints(crosspoint2, parent2, dnode2, dnodeparent2);
		index = dnode1->addno;
		checkAdjoint(p, dnodeparent1, i, index, t, found1, dnode2);

		if (found1) {
			index = dnode2->addno;
			checkAdjoint(p, dnodeparent2, i, index, t, found2, dnode1);
			if (found2) {//they are compatible
				act = TRUE;
			    calculateSize(exroot1, exroot2, root1, dnode1, root2, dnode2, act, size1, size2);

				if (exroot1) {
					setSize(ch, dnode1, size1);
				}
				if (exroot2) {
					setSize(ch, dnode2, size2);
				}

				setAct(fairsize, size1, size2, diff, act, parent1, parent2);
			    if (act) {//swaping
					swapTree(temp1, dnode1, dnode2, l, temp2);

					if (exroot1 && exroot2) {
						beforeSwapping(ch, dnode1, ch2, dnode2, temp1, l, temp2);
						rootswapping++;
					} else if (exroot1) {
						beforeSwapping(dnode1, ch2, dnode2, ch);
						rootswapping++;
					} else if (exroot2) {
						beforeSwapping(dnode2, ch, dnode1, ch2);
						// rootswapping++;
					}

				    recomputeSizeNBranch(child1, parent1, size2, size1, dnode1);
				    recomputeSizeNBranch(child2, parent2, size1, size2, dnode2);

					return TRUE;
				}
			}
		}
		j++; //Next attempt
	}
	return FALSE;
}
