//************************************************************************************
//TAG3P VERSION 3.2 @copyright by Nguyen Xuan Hoai                                   *
//This file contains individual,population representations and genetic operators     *
//************************************************************************************
#ifndef __GENETIC__
#define __GENETIC__

#include<iostream>
#include<cstdlib>
#include<cstring>
#include<cassert>
#include<cmath>
#include "../gconst.h"
#include "../random/random.h"
#include "../stree/stree.h"
#include "../lexicon/lexicon.h"
#include "../adtable/adtable.h"
#include "../dtree/dtree.h"

#ifdef _SP_MODE
#include "../smart_ptr.h"
#endif
using namespace std;
//Declaration for individual and population
struct _individual {
	#ifdef _SP_MODE
	sp_dnode chrom;//chromosome of individual
	#else
	dnode *chrom;//chromosome of individual
	#endif
	SIZE size;//size of individual
	double branch;//branching of individual
	double fitness;//fitness of individual
};
typedef struct _individual individual;
#ifdef _SP_MODE
typedef boost::shared_array<_individual> sa_individual;
extern sa_individual oldpop, newpop;//New and old populations
#else
extern individual *oldpop, *newpop;//New and old populations
#endif
extern unsigned int poplen; //population length
extern double pcross, pinsert, pdelete, preplace, pmutate, prep, ptran;//probability for genetic operations
extern long ncross, ninsert, ndelete, nreplace, nmutate, nrep, ntran;//statistics
extern BOOL _cut;
extern long rootswapping;
//*********************
//Copy an individual  *
//*********************
individual CopyIndividual(individual t, BOOL copychrom);
//*****************************************************
//Decode a individual to a derivation tree            *
//*****************************************************
stree Decode(individual v, BOOL stochastic);
//*************************
//Display an individual   *
//*************************
void DisplayIndividual(individual v);
//*********************************************************
//Initialize a  shaped tree with a precified size(>=1)    *
//start tree=TRUE-start with alpha tree; FALSE- start with*
//beta tree; Supplied adjoining address (irrelevant when  *
//start with alpha tree                                   *
//*********************************************************
#ifdef _SP_MODE
sp_dnode GenShapeTree(SIZE &size, SIZE starttree, BYTE addno, BOOL shape);
#else
dnode *GenShapeTree(SIZE &size, SIZE starttree, BYTE addno, BOOL shape);
#endif
//***************************
//Uniform Initialization    *
//***************************
void UniformInit(unsigned int minl, unsigned int maxl);
//*****************************************************
//Ramp Initialization with a percentage of full tree  *
//*****************************************************
void RampInit(unsigned int minl, unsigned int maxl, double percentage);//(minl>=2);
//Initialization Procedure
void Initialization(unsigned int minl, unsigned int maxl, unsigned int type,
		double percentage);
//************************************************
//This function computes Adjusted Fitness        *
//This assumes that the fitness of an individual *
//is already standardized (it means the smaller  *
//the better)                                    *
//************************************************
void AdjustFitness();
//***********************************************
//This function computes normalized fitness it  *
//supposes that the fitness of an individual is *
//already adjusted (it means between 0 - 1 and  *
//The bigger the better)                        *
//***********************************************
void NormalizeFitness();
//*************************************************
//Selections works on Normalized Fitness          *
//*************************************************
//*************************
//Proportionate Selection  *
//*************************
unsigned int ProSelect();
//*********************
//Tournament Selection*
//*********************
unsigned int TourSelect(unsigned int size);
//Linear Ranking Selection

//**************************************************
//Crossover Section                                *
//**************************************************
//*********************************************
//check if a tree can be adjoined at its root *
//*********************************************
BOOL RootAdjoinability(SIZE treeid);
//****************************************************
//Check if a node is currently adjoined at its root  *
//****************************************************
#ifdef _SP_MODE
BOOL RootAdjoined(sp_dnode p);
#else
BOOL RootAdjoined(dnode *p);
#endif
//*******************************
//Crossover point distribution  *
//******************************
#ifdef _SP_MODE
extern boost::shared_array<double> _pointdis;
void DNodeProb(sp_dnode t);
#else
extern double *_pointdis;
void DNodeProb(dnode *t);
#endif
extern unsigned int _dnodeindex1;
unsigned int GetCrossPoint(individual p);
//This function implement subtree crossover by swaping two       *
//subtrees. The adjoining addresses (contexts) are maintained    *
//****************************************************************
BOOL SubTreeSwap(individual parent1, individual parent2, individual &child1,
		individual &child2, BOOL fairsize, unsigned int diff);
//****************************************************************
//This function implement subtree crossover by swaping two       *
//subtrees, open addresses are included as Null Nodes            *
//****************************************************************
BOOL SubTreeSwap2(individual &parent1, individual &parent2, individual &child1,
		individual &child2, BOOL fairsize, unsigned int diff, double prob);
//*******************************************************************************
//The same as a bove but can exchange subcode of type 1 only -                  *
//*******************************************************************************
BOOL OldSubTreeSwap(individual parent1, individual parent2, individual &child1,
		individual &child2, BOOL fairsize, unsigned int diff);
//*******************************************************
//version of Oldsubtree Swap but permit NULL address    *
//*******************************************************
BOOL OldSubTreeSwap2(individual &parent1, individual &parent2,
		individual &child1, individual &child2, BOOL fairsize,
		unsigned int diff, double prob);
//****************************************************************
//This function implement subtree crossover by swaping children  *
// of two nodes. The adjoing addresses (contexts) are maintained *
//****************************************************************
BOOL SubSubTreeSwap(individual parent1, individual parent2, individual &child1,
		individual &child2, BOOL fairsize, unsigned int diff);
//******************************************************************
//This function implement subtree crossover by swaping children    *
//of two nodes. The adjoing addresses (contexts) are notmaintained *
//******************************************************************
//BOOL SubSubTreeSwap2(unsigned int parent1,unsigned int parent2,individual *child1,individual *child2,BOOL fairsize,unsigned int diff);//{
//}
//**************************************************
//Mutation Section                                 *
//**************************************************************
//This function will replace subtree by randomly               *
//generated one; fairsize=TRUE the difference in size must be  *
//between 0 and diff; shape=TRUE means FULL otherwise RANDOM   *
//use size to specidied the size you want                      *
//The address (context) is main tained                         *
//**************************************************************
BOOL ReplaceSubTree(individual &child, SIZE size, BOOL shape, BOOL fairsize,
		unsigned int diff);
//***********************************
//ReplaceSubtree with NULL Nodes    *
//***********************************
BOOL ReplaceSubTree2(individual &child, SIZE size, BOOL shape, BOOL fairsize,
		unsigned int diff, double prob);//
//**************************************************************
//This function will replace subtree by randomly               *
//generated one; fairsize=TRUE the difference in size must be  *
//between 0 and diff; shape=TRUE means FULL otherwise RANDOM   *
//use size to specidied the size you want                      *
//The address (context) is not maintained                      *
//**************************************************************
//BOOL ReplaceSubTree(individual &child,SIZE size, BOOL shape,BOOL fairsize,unsigned int diff);//{
//}
//*********************************************************
//this function replace a node by another compatible node *
//*********************************************************
BOOL PointReplace(individual &child);
//***************************
//Directed Point Mutation   *
//***************************
#ifdef _SP_MODE
extern boost::shared_array<SIZE> _replacelist;
#else
extern SIZE *_replacelist;
#endif
BOOL DirectedPointReplace(individual &child);
//**************************
//      Lexicon Mutation   *
//**************************
#ifdef _SP_MODE
void DNodeLexMutation(sp_dnode t);
#else
void DNodeLexMutation(dnode *t);
#endif
//**************************
//Point Lexicon Mutation   *
//**************************
BOOL PointLexMutation(individual &child);
//******************
//Lexicon Mutation *
//******************
BOOL LexMutation(individual &child);
//*****************
//SIZE MUTATIONS  *
//*****************
BOOL Delete(individual &child, SIZE size);
BOOL Insert(individual &child, SIZE size, BOOL shape);
//******************************************************************
//One Step Mutation: either change value or add or delete one node *
//******************************************************************
BOOL OneStepMutation(individual &child, BOOL shape);

//*****************************************************
//Relocation, Transposition and Replication Operators *
//*****************************************************
//****************************
//relocation of a subtree    *
//****************************
BOOL Relocate(individual &child);

//Replication
BOOL Replicate(individual &child, BOOL recursive);
#endif
