//***********************************************************************
//This file contains declarations and operations for lexicons in LTAGs  *
//For TAG3P version 3.0. Programmed by NXH			        *
//***********************************************************************
#ifndef __LEXICON__
#define __LEXICON__
#include<iostream>
#include<cstring>
#include<iomanip>
#include<cmath>
#include<cstdlib>
#include<cassert>
#include "../random/random.h"
#include "../stree/stree.h"

#ifdef _SP_MODE
#include "../smart_ptr.h"
#endif

#define MAXLEX 20




using namespace std;

typedef unsigned char LEXSIZE;
struct _lexeme {

#ifdef _SP_MODE
	sp_node lex;
#else
	node *lex;
#endif

	double prob;
};
typedef struct _lexeme lexeme;
struct _lexicon {
	lexeme lexemes[MAXLEX];
	LEXSIZE card;
	char name[MAXNAME];
};
typedef struct _lexicon lexicon;

#ifdef _SP_MODE
typedef boost::shared_array<_lexicon> sa_lexicon;
#endif


#ifdef _SP_MODE
extern sa_lexicon vocab;
#else
extern lexicon *vocab;
#endif

extern LEXSIZE lexnum;//number of lexicons
struct _lnode {
	LEXSIZE lexiconid;
	LEXSIZE lexemeid;
	double value;

#ifdef _SP_MODE
	boost::shared_ptr<_lnode> next;
#else
	struct _lnode *next;
#endif

};

#ifdef _SP_MODE
typedef boost::shared_ptr<_lnode> sp_lnode;
#endif

typedef struct _lnode lnode;
struct _llist {
#ifdef _SP_MODE
	sp_lnode head;
#else
	lnode *head;
#endif
};

typedef struct _llist llist;
void IniLList(llist &l);
BOOL EmptyLList(llist l);
void DeleteLList(llist &l);
void DeleteHeadLList(llist &l);
void AddTailLList(llist &l, LEXSIZE lexiconid, LEXSIZE lexemeid, double value);
llist CopyLList(llist l);

//**********************************
//Normalise lexeme distribution    *
//**********************************
void NormaliseLD(LEXSIZE lexiconid);
void NormaliseVD(void);

//*****************************************************************
//Get a lexeme for a lexicon according to the lexeme distribution *
//*****************************************************************
#ifdef _SP_MODE
sp_node GetLexeme(LEXSIZE lexid);
#else
node *GetLexeme(LEXSIZE lexid);
#endif
LEXSIZE GetLexemeID(LEXSIZE lexid);

extern double (*GetERconst)(char *name);

double DefaultConstant(char *name);

//********************************************************
//Build the list of lexicon for substitition from a tree *
//********************************************************
#ifdef _SP_MODE
void BuildLListNode(sp_node s, llist &l);
llist BuildLList(sp_node t);
#else
void BuildLListNode(node *s, llist &l);
llist BuildLList(node *t);
#endif

//*************************
//Initialisation of vocab;*
//*************************

//*********************************************************************
//Substitute a s-tree with lexicons according to lexeme distributions *
//*********************************************************************
#ifdef _SP_MODE
void SubstituteLexicon(sp_node s);
void DSubstituteLexiconNode(sp_node s, llist &l);
void DSubstituteLexicon(sp_node s, llist l);
#else
void SubstituteLexicon(node *s);
void DSubstituteLexiconNode(node *s, llist &l);
void DSubstituteLexicon(node *s, llist l);
#endif

//**********************************************
//Substitute a s-tree with lexicon from a list *
//**********************************************

//*****************************
//Initialise the vocabulary   *
//*****************************
#endif
