/*
 * MarkingDtree1.cpp
 *
 *  Created on: 2010. 5. 15.
 *      Original Author: Nguyen Xuan Hoai
 *      Modifications: Structural Complexity Lab, Seoul National Univ.
 */
#include "prune.h"

#ifdef _SP_MODE
void doMarking(SIZE & size, sp_dnode & t, double & f, individual & v, BOOL & type, BOOL flag)
#else
void doMarking(SIZE & size, dnode * & t, double & f, individual & v, BOOL & type, BOOL flag)
#endif
{
	if (_activesize - size >= MINSIZE) { //only do the marking if the result bigger than MIN SIZE
		if (flag) {
			t->id = -t->id;//temporary marked off
		} else {
			t->id = -(t->id + numetree); //temporarily marked for subcode type 2
		}

		f = ComputeIndividualRF(v); //compute the new fitness

		if (fabs(f - _rawfitness) <= _variance) {//if the change is insignificant
			_activesize -= size;
			type = TRUE;
		} else {
			if (flag) {
				t->id = -t->id; //turns it back on
			} else {
				t->id = -(t->id + numetree);
			}
		}
	}
}

#ifdef _SP_MODE
void MarkingDtree1(sp_dnode t, individual &v) { //use for both types of sub-code added in version 3.1
	sp_dnode p;
#else
void MarkingDtree1(dnode *t, individual &v) { //use for both types of sub-code added in version 3.1
	dnode *p;
#endif
	double f;
	SIZE size1, size2;
	BOOL type1OK, type2OK;

	if (t) {
		size1 = GetSizeDNode(t, TRUE);
		type1OK = FALSE;
		type2OK = FALSE;
		doMarking(size1, t, f, v, type1OK, true);

		if (!type1OK) {//if type1 subcode is not marked then try type2
			if (t != v.chrom) {//not the root
				if (RootAdjoined(t)) {
					p = t->children;
					while (p->sibling) {
						p = p->sibling;
					}
					size2 = GetSizeDNode(p, TRUE);
					size2 = size1 - size2;
					doMarking(size2, t, f, v, type2OK, false);
				}
			}

			//try the children
			if (!type2OK) {
				p = t->children;
				while (p) {
					MarkingDtree1(p, v);
					p = p->sibling;
				}
			} else {
				p = t->children;
				while (p->sibling) {
					p = p->sibling;
				}
				MarkingDtree1(p, v);
			}
		}
	}
}
