#ifndef __STREE__
#define __STREE__
//**********************************************************************
//This file contains declaration and utility to manipulate sentencial                                      *
//trees. Programmed by NXH. (version 3.0)                                                                        *
//**********************************************************************
#include<iostream>
#include<cstring>
#include<cassert>
#include<cstdlib>
#include<ctype.h>
#include <stdio.h>
#include "../gconst.h"

#ifdef _SP_MODE
#include "../smart_ptr.h"
#endif

using namespace std;

#define VOIDVALUE -999999999.2732897

#ifdef _SP_MODE
// Debug mode
// every pointer is changed to shared_ptr or shared_array
// except char *name and function pointer

//*********************************************************************
//Declarations and Functions for Node trees                           *
//*********************************************************************
struct _node {
	char name[MAXNAME];
	double value;
	boost::shared_ptr<_node> sibling;
	boost::shared_ptr<_node> children;
};
typedef struct _node node;

typedef boost::shared_ptr<_node> sp_node;
//***************************
//Make a Node               *
//***************************
sp_node MakeNode(char *nname, double value);
//*****************************
//Delete a subtree            *
//*****************************
void DeleteNode(sp_node t);

//**********************************
//Copy a subtree                   *
//**********************************
extern sp_node _oldfoot;//this variable contains the foot node of the source tree
extern sp_node _newfoot;//this variable contains the footnode of the newtree
sp_node CopyNode(sp_node s);

//***********************************
//Get the height of a subtree       *
//***********************************
unsigned int GetHeightNode(sp_node t);

//*************************************
//Compare two subtree                 *
//*************************************
BOOL NodeCmp(sp_node t, sp_node s);

//*********************************************************
//Travel a Tree in Preorder,Inorder and Postoder          *
//*********************************************************
extern void (*_doit)(sp_node n);
void Preorder(sp_node t);

//Travel a Tree in Inoder
void Inorder(sp_node t);

//Travel a Tree in Postorder
void Postorder(sp_node t);
//***********************************
//From a string to STree            *
//***********************************
extern char _sbuff[MAXSTRING];
extern unsigned int _curchar;
extern sp_node _sfoot;
sp_node StringToTree();

//*************************************
//From a tree to string with brackets *
//*************************************
void TreeToString(sp_node r, sp_node f);

//*****************************************************
//To check if node p is in the subtree stermed from q *
//*****************************************************
BOOL InNode(sp_node p, sp_node q);

//*******************************************************
//This function compare two node it returns             *
//+1 if p is in q,-1 if q is in p, and 0 if incomparable*
//*******************************************************
int NodeOrder(sp_node p, sp_node q);

//********************************
//Compute the size of a subtree  *
//********************************
extern unsigned int _nodecount;
extern BOOL _cntleave;
void NodeCount(sp_node t);
SIZE GetSizeNode(sp_node t, BOOL countleave);

//************************************************************************
//Declarations and Global Variables for Strees                           *
//************************************************************************
struct _stree {
	sp_node root;
	sp_node foot;
};
typedef struct _stree stree;
typedef boost::shared_ptr<_stree> sp_stree;
typedef boost::shared_array<_stree> sa_stree;
extern sa_stree etree;
extern SIZE numetree;
extern SIZE numalpha, numbeta;

//*********************************
//Initialize a Null tree          *
//*********************************
stree InitEmptyTree();
stree InitTree(const char *st);

//*********************************
//Delete a tree from memory       *
//*********************************
void DeleteTree(stree s);
void Display(stree s);

//*********************************
//Check whether a tree is empty   *
//*********************************
BOOL IsEmpty(stree s);

//*****************************************
//Check whether a tree is an alpha tree   *
//*****************************************
BOOL IsAlpha(stree s);

//***************************
//Get height of a tree      *
//***************************
unsigned int GetHeight(stree s);
SIZE GetSize(stree s, BOOL leave);

//****************************
//Copy a tree                *
//****************************
stree CopyTree(stree s);

//****************************
//Compare two trees          *
//****************************
BOOL CmpTree(stree t, stree s);

//**************************************************************
//Adjoin a list of beta trees to an alpha or betatree          *
//**************************************************************
stree AdjointTree(stree beta, stree gama, sp_node add, sp_node parent);





#else
// original xuan's code
//
//
//*********************************************************************
//Declarations and Functions for Node trees                           *
//*********************************************************************
struct _node {
	char name[MAXNAME];
	double value;
	struct _node *sibling, *children;
};
typedef struct _node node;

//***************************
//Make a Node               *
//***************************
node *MakeNode(char *nname, double value);
//*****************************
//Delete a subtree            *
//*****************************
void DeleteNode(node *t);

//**********************************
//Copy a subtree                   *
//**********************************
extern node *_oldfoot;//this variable contains the foot node of the source tree
extern node *_newfoot;//this variable contains the footnode of the newtree
node *CopyNode(node *s);

//***********************************
//Get the height of a subtree       *
//***********************************
unsigned int GetHeightNode(node *t);

//*************************************
//Compare two subtree                 *
//*************************************
BOOL NodeCmp(node *t, node *s);

//*********************************************************
//Travel a Tree in Preorder,Inorder and Postoder          *
//*********************************************************
extern void (*_doit)(node *n);
void Preorder(node *t);

//Travel a Tree in Inoder
void Inorder(node *t);

//Travel a Tree in Postorder
void Postorder(node *t);
//***********************************
//From a string to STree            *
//***********************************
extern char _sbuff[MAXSTRING];
extern unsigned int _curchar;
extern node *_sfoot;
node *StringToTree();

//*************************************
//From a tree to string with brackets *
//*************************************
void TreeToString(node *r, node *f);

//*****************************************************
//To check if node p is in the subtree stermed from q *
//*****************************************************
BOOL InNode(node *p, node *q);

//*******************************************************
//This function compare two node it returns             *
//+1 if p is in q,-1 if q is in p, and 0 if incomparable*
//*******************************************************
int NodeOrder(node *p, node *q);

//********************************
//Compute the size of a subtree  *
//********************************
extern unsigned int _nodecount;
extern BOOL _cntleave;
void NodeCount(node *t);
SIZE GetSizeNode(node *t, BOOL countleave);

//************************************************************************
//Declarations and Global Variables for Strees                           *
//************************************************************************
struct _stree {
	node *root;
	node *foot;
};
typedef struct _stree stree;
extern stree *etree;
extern SIZE numetree;
extern SIZE numalpha, numbeta;

//*********************************
//Initialize a Null tree          *
//*********************************
stree InitEmptyTree();
stree InitTree(const char *st);

//*********************************
//Delete a tree from memory       *
//*********************************
void DeleteTree(stree s);
void Display(stree s);

//*********************************
//Check whether a tree is empty   *
//*********************************
BOOL IsEmpty(stree s);

//*****************************************
//Check whether a tree is an alpha tree   *
//*****************************************
BOOL IsAlpha(stree s);

//***************************
//Get height of a tree      *
//***************************
unsigned int GetHeight(stree s);
SIZE GetSize(stree s, BOOL leave);

//****************************
//Copy a tree                *
//****************************
stree CopyTree(stree s);

//****************************
//Compare two trees          *
//****************************
BOOL CmpTree(stree t, stree s);

//**************************************************************
//Adjoin a list of beta trees to an alpha or betatree          *
//**************************************************************
stree AdjointTree(stree beta, stree gama, node *add, node *parent);
#endif
#endif
