/*
 * Compute.cpp
 *
 *  Created on: 2010. 5. 12.
 *      Original Author: Nguyen Xuan Hoai
 *      Modifications: Structural Complexity Lab, Seoul National Univ.
 */

#include "symregr.h"
#ifdef _SP_MODE
double Compute(sp_node st, double x) {

	sp_node p;
#else
double Compute(node *st, double x) {

	node *p;
#endif
	double l, r;

	if (!strcmp(st->name, "X")) {
		return x;
	} else if (!strcmp(st->name, "R1") || !strcmp(st->name, "R2")) { //st->name="EXP"
		return st->value;
	} else {
		p = st->children;
		if (!strcmp(p->name, "EXP")) {
			l = Compute(p, x);//compute the left branch
			p = p->sibling;
			r = Compute(p->sibling, x);//compute the right branch
			p = p->children;//get the operator
			if (!strcmp(p->name, "add")) {
				return PVAL(r+l);
			} else if (!strcmp(p->name, "sub")) {
				return PVAL(l-r);
			} else if (!strcmp(p->name, "mul")) {
				return PVAL(r*l);
			} else if (r != 0) { //protected division
				return PVAL(l/r);
			} else {
				return 1;
			}
		} else if (!strcmp(p->name, "PRE")) {
			r = Compute(p->sibling, x); //compute exp inside function
			p = p->children;
			if (!strcmp(p->name, "sin")) {
				return sin(r);
			} else if (!strcmp(p->name, "cos")) {
				return cos(r);
			} else if (!strcmp(p->name, "ep")) {
				return PVAL(exp(r));
			} else if (r != 0) { //protected logarithm
				return log(fabs(r));
			} else {
				return 0;
			}
		} else {//VAR
			return Compute(p->children, x);
		}
	}
}
