//************************************************************************
//This module contains fitness case and functions to compute raw fitness *
//as well as standardized fitness for Symbolic Regression Problem        *
//Programmed by NXH.                                                     *
//************************************************************************
#ifndef __SYMBOLICREG__
#define __SYMBOLICREG__

#include<iostream>
#include<cstring>
#include<cstdio>
#include<cassert>
#include<cmath>
#include<cstdlib>
#include "../gconst.h"
#include "../random/random.h"
#include "../stree/stree.h"
#include "../lexicon/lexicon.h"
#include "../adtable/adtable.h"
#include "../dtree/dtree.h"

#define NUMFITCASE 20
#define INFPLUS  exp(700)
#define INFMINUS -exp(700)//#define EXPOVL 650
#define PVAL(y) ((y==HUGE_VAL)?(INFPLUS):((y==-HUGE_VAL)?(-INFMINUS):(y)))

//****************************************
//Fitness cases for symbolic regression  *
//They are sampled between 0  and 1      *
//Seed for sampling 12345                *
//****************************************
struct _sample {
	double x;
	double y;
};
typedef struct _sample sample;

extern sample fitcase[NUMFITCASE];
extern BOOL SuccPredicate;

void SetFitCase();
//This function is for getting ephemeral constants
double ERGenerate(char *st);
//********************************************************************
//This function compute the function (in tree form) with given input *
//********************************************************************
#ifdef _SP_MODE
double Compute(sp_node st, double x);
#else
double Compute(node *st, double x);
#endif
//*********************************************
//Compute the raw fitness for an invidual     *
//*********************************************
double ComputeRF(stree st);
void BuildVocab();
//****************************************************************
//This function initializes elementary trees and adjoining table *
//****************************************************************
void LoadETree();
#ifdef _SP_MODE
sp_node Stree2ParseTree(sp_node st);
#else
node *Stree2ParseTree(node *st);
#endif

#endif
