/*
 * Util.cpp
 *
 *  Created on: 2010. 5. 25.
 *      Original Author: Nguyen Xuan Hoai
 *      Modifications: Structural Complexity Lab, Seoul National Univ.
 */

#include "util.h" 

#include "../dtree/dtree.h"
#include "../genetic/genetic.h"

#ifdef _SP_MODE
void insertBeta(sp_dnode q, sp_dnode p, BYTE addno, sp_dnode t)
#else
void insertBeta(dnode * q, dnode * p, BYTE addno, dnode * t)
#endif
{
	//insert in the increasing oder of address
	BOOL found = FALSE;
	q = p;
	while (p && !found) {
		if (p->addno > addno) {
			found = TRUE;
		} else {
			q = p;
			p = p->sibling;
		}
	}
	if (!found) {
		//insert at the end
		q->sibling = t;
	} else if (p == q) {
		//found at first child
		_opendnode->children = t;
		t->sibling = p;
	} else {
		//insert before p
		q->sibling = t;
		t->sibling = p;
	}
}

#ifdef _SP_MODE
void swapTree(unsigned int temp1, sp_dnode & dnode1, sp_dnode & dnode2,
		llist l, sp_dnode temp2)
#else
void swapTree(unsigned int temp1, dnode * & dnode1, dnode * & dnode2, llist l, dnode * temp2)
#endif
{
	//swaping tree id first
	temp1 = dnode1->id;
	dnode1->id = dnode2->id;
	dnode2->id = temp1;
	//then swaping lexlist
	l = dnode1->lexlist;
	dnode1->lexlist = dnode2->lexlist;
	dnode2->lexlist = l;
	//then swaping the children
	temp2 = dnode1->children;
	dnode1->children = dnode2->children;
	dnode2->children = temp2;
}

void setAct(BOOL fairsize, SIZE & size1, SIZE & size2, unsigned int diff,
		BOOL & act, individual & parent1, individual & parent2) {
	if (fairsize) {//fairsize crossover
		//check if the difference is small enough
		if (ABSDIFF(size1,size2) > diff) {
			act = FALSE;
		}
	}
	//check the size limit
	if ((ABSDIFF(parent1.size+size2,size1) < MINSIZE)
			|| (ABSDIFF(parent2.size+size1,size2) < MINSIZE)) {
		act = FALSE;
	}
	if (((ABSDIFF(parent1.size+size2,size1) > MAXSIZE)
			|| (ABSDIFF(parent2.size+size1,size2) > MAXSIZE)) && !_cut) {
		act = FALSE;
	}
}

void checkAndSetAct(BOOL fairsize, SIZE & size1, SIZE & size2,
		unsigned int diff, BOOL & act, individual & parent1,
		individual & parent2) {
	if (fairsize) {//fairsize crossover
		//check if the difference is small enough
		if (ABSDIFF(size1, size2) > diff) {
			act = FALSE;
		}
	}
	//check the size limit
	if ((ABSDIFF(parent1.size+size2, size1) < MINSIZE)
			|| (ABSDIFF(parent2.size+size1, size2) < MINSIZE)) {
		act = FALSE;
	}
	if (((parent1.size - size1 + size2 > MAXSIZE) || (parent2.size - size2
			+ size1 > MAXSIZE)) && !_cut) {
		if ((ABSDIFF(parent1.size+size2, size1) < MINSIZE)
				|| (ABSDIFF(parent2.size+size1, size2) < MINSIZE)) {
			act = FALSE;
		}
	}

	if (parent2.size >= MAXSIZE) {
		act = FALSE;
	}
}

#ifdef _SP_MODE
void checkAdjoint(sp_address p, sp_dnode & dnodeparent, unsigned int i,
		BYTE & addindex, sp_snode t, BOOL & found, sp_dnode & dnode)
#else
void checkAdjoint(address * p, dnode * & dnodeparent, unsigned int i, BYTE & addindex, snode * t, BOOL & found, dnode * & dnode)
#endif
{
	//check if dnode could adjoint to dnodeparent
	p = adtable[dnodeparent->id].head;
	i = 0;
	while (i < addindex) {
		p = p->next;
		i++;
	}

	t = (p->source).head;
	found = FALSE;
	while (t && !found) {
		if (t->id == dnode->id) {
			found = TRUE;
		} else {
			t = t->next;
		}
	}
}

#ifdef _SP_MODE
void checkAdjoint(sp_snode & t, sp_address & p, BOOL & found, sp_dnode & q)
#else
void checkAdjoint(snode * & t, address * & p, BOOL & found, dnode * & q)
#endif
{
	//find the tree
	t = (p->source).head;
	found = FALSE;
	while (t && !found) {
		if (t->id == q->id) {
			found = TRUE;
		} else {
			t = t->next;
		}
	}
}

#ifdef _SP_MODE
void checkAdjoint(sp_address & p, sp_dnode & dnode, unsigned int i,
		sp_dnode & q, sp_snode & t, BOOL & found)
#else
void checkAdjoint(address * & p, dnode * & dnodearg, unsigned int i, dnode * & q, snode * & t, BOOL & found)
#endif
{
	p = adtable[dnodearg->id].head;
	i = 0;
	//find the address
	while (i < q->addno) {
		p = p->next;
		i++;
	}
	checkAdjoint(t, p, found, q);
}

#ifdef _SP_MODE
void findOpenNode(unsigned int opendnodeno, individual & child,
		sp_dnode & dnode)
#else
void findOpenNode(unsigned int opendnodeno, individual & child, dnode * & dnode)
#endif
{
	//find open node
	_opendnodeno = IRandom(1, opendnodeno);
	_countorfind = FALSE;
	_opendnode = NULL;
	_doitdnode = CountOpenDNode;
	PreorderDNode(child.chrom); //results is in _opendnode
	dnode = _opendnode;
}

#ifdef _SP_MODE
void getCrossOverPoints(unsigned int crosspoint, individual & parent,
		sp_dnode & dnode, sp_dnode & dnodeparent)
#else
void getCrossOverPoints(unsigned int crosspoint, individual & parent, dnode * & dnodearg, dnode * & dnodeparent)
#endif
{
	crosspoint = IRandom(2u, parent.size); //exclude the root
	LocateDNode(crosspoint, parent.chrom, NULL);
	dnodearg = _idnode;
	dnodeparent = _idnodeparent;
}

#ifdef _SP_MODE
void getCrossoverPoints(unsigned int crosspoint, individual & parent,
		sp_dnode & dnode)
#else
void getCrossoverPoints(unsigned int crosspoint, individual & parent, dnode * & dnode)
#endif
{
	crosspoint = IRandom(1, parent.size); //include the root
	LocateDNode(crosspoint, parent.chrom, NULL);
	dnode = _idnode;
}

#ifdef _SP_MODE
void recomputeSizeNBranch(individual & child, individual parent, SIZE & size2,
		SIZE & size1, sp_dnode dnode)
#else
void recomputeSizeNBranch(individual & child, individual parent, SIZE & size2, SIZE & size1, dnode * dnode)
#endif
{
	//recompute size and branch
	child.chrom = parent.chrom;
	child.size = ABSDIFF(parent.size+size2,size1);

	if (child.size > MAXSIZE) { //randomly cut
		SIZE k;
		for (k = MAXSIZE; k < child.size; k++) {
			DeleteBeta(dnode);
		}
		child.size = MAXSIZE;
	}

	child.branch = GetAVGDNode(child.chrom);
}

void recomputeSizeNBranch(individual & child, unsigned int & l) {
	if (child.size > MAXSIZE) {//randomly cut
		for (l = MAXSIZE; l < child.size; l++) {
			DeleteBeta(child.chrom);
		}
		child.size = MAXSIZE;
	}
	child.branch = GetAVGDNode(child.chrom);
}

void recomputeSizeNBranch(individual & child, individual & parent,
		SIZE & size2, SIZE & size1) {
	//recompute size and branch
	child.chrom = parent.chrom;
	child.size = ABSDIFF(parent.size+size2, size1);
	child.branch = GetAVGDNode(child.chrom);
}

void copyIndividual(individual & tmp, individual & node1, individual & node2) {
	tmp = CopyIndividual(node1, FALSE);
	node1 = CopyIndividual(node2, FALSE);
	node2 = CopyIndividual(tmp, FALSE);
}

#ifdef _SP_MODE
void increaseOrderInsert(BOOL found, sp_dnode tp2, sp_dnode tp1, BYTE addindex,
		sp_dnode & dnode1, sp_dnode dnodeparent2)
#else
void increaseOrderInsert(BOOL found, dnode * tp2, dnode * tp1, BYTE addindex, dnode * & dnode1, dnode * dnodeparent2)
#endif
{
	//insert in an increasing order of addresses
	found = FALSE;
	tp2 = tp1;
	while (tp1 && !found) {
		if (tp1->addno > addindex) {
			found = TRUE;
		} else {
			tp2 = tp1;
			tp1 = tp1->sibling;
		}
	}
	if (!found) {
		//insert at the end
		tp2->sibling = dnode1;
	} else if (tp1 == tp2) {
		//found at first child
		dnodeparent2->children = dnode1;
		dnode1->sibling = tp1;
	} else {
		//insert before tp1
		tp2->sibling = dnode1;
		dnode1->sibling = tp1;
	}
}

#ifdef _SP_MODE
void swapParents(sp_dnode tp1, sp_dnode dnodeparent1, sp_dnode & dnode1,
		BYTE addindex, sp_dnode dnodeparent2, BOOL found, sp_dnode tp2)
#else
void swapParents(dnode * tp1, dnode * dnodeparent1, dnode * & dnode1,BYTE addindex, dnode * dnodeparent2, BOOL found, dnode * tp2)
#endif
{
	//swapping the two
	//cout<<"Entering Act!"<<endl;
	//first deattach the dnode1
	tp1 = dnodeparent1->children;
	if (tp1 == dnode1) {
		//dnode1 is the first children
		dnodeparent1->children = tp1->sibling;
	} else {
		while (tp1->sibling != dnode1) {
			tp1 = tp1->sibling;
		}
		tp1->sibling = dnode1->sibling;
	}

	dnode1->sibling = NULL;
	dnode1->addno = addindex;
	//attach dnode1 to the Null address of Dnode2 using addindex
	tp1 = dnodeparent2->children;
	if (tp1 == NULL) {
		dnodeparent2->children = dnode1;
	} else {
		increaseOrderInsert(found, tp2, tp1, addindex, dnode1, dnodeparent2);
	}
}

#ifdef _SP_MODE
void findNInsert(BOOL found, sp_dnode q, sp_dnode dnode2, sp_dnode p,
		sp_dnode dnode1)
#else]
void findNInsert(BOOL found, dnode * q, dnode * dnode2, dnode * p, dnode * dnode1)
#endif
{
	if (found) {
		q = dnode2->children;
		if (q == p) {//find in first place
			dnode1->sibling = q;
			dnode2->children = dnode1;
		} else {
			while (q->sibling != p) {
				q = q->sibling;
			}
			q->sibling = dnode1;
			dnode1->sibling = p;
		}
	} else {//insert at the end
		q = dnode2->children;
		while (q->sibling) {
			q = q->sibling;
		}
		q->sibling = dnode1;
		dnode1->sibling = NULL;
	}
}

